<?php

// a PHP client library for pubsubhubbub
// as defined at http://code.google.com/p/pubsubhubbub/
// written by Josh Fraser | joshfraser.com | josh@eventvue.com
// Released under Apache License 2.0

class Publisher {
    
    var $hub_url;
    var $last_response;
    
    // create a new Publisher
    function __construct($hub_url) {
        
        if (!isset($hub_url))
            throw new Exception('Please specify a hub url');
        
        if (!preg_match("|^https?://|i",$hub_url)) 
            throw new Exception('The specified hub url does not appear to be valid: '.$hub_url);
            
        $this->hub_url = $hub_url;
    }

    // accepts either a single url or an array of urls
    function publish_update($topic_urls, $http_function = false) {
        if (!isset($topic_urls))
            throw new Exception('Please specify a topic url');
        
        // check that we're working with an array
        if (!is_array($topic_urls)) {
            $topic_urls = array($topic_urls);
        }
        $post_string = "hub.mode=publish";
        // check each topic url 
        foreach ($topic_urls as $topic_url) {

            // lightweight check that we're actually working w/ a valid url
            if (!preg_match("|^https?://|i",$topic_url)) 
                throw new Exception('The specified topic url does not appear to be valid: '.$topic_url);
                $post_string .= "&hub.url=".urlencode($topic_url);
        }

        return $this->http_post($this->hub_url, $post_string);
    }

    // returns any error message from the latest request
    function last_response() {
        return $this->last_response;
    }
    
    // default http function that uses curl to post to the hub endpoint
    function http_post($url, $post_string) {
        if (function_exists('curl_init')) {
            $options = array(CURLOPT_URL => $url,
                         CURLOPT_POST => true,
                         CURLOPT_POSTFIELDS => $post_string,
                         CURLOPT_USERAGENT => "PubSubHubbub-Publisher-PHP/1.0");

            $ch = curl_init();
            curl_setopt_array($ch, $options);

            $response = curl_exec($ch);
            $this->last_response = $response;
            $info = curl_getinfo($ch);

            curl_close($ch);
            
            $responseCode = $info['http_code'];
 
        } else {
            @include_once 'HTTP/Request.php';
            if (!class_exists('HTTP_Request')) {
                return false;
            }
            $sender = &new HTTP_Request($url);
            $sender->setMethod(HTTP_REQUEST_METHOD_POST);

            $sender->addRawQueryString($post_string);
            
            
            $sender->addHeader('Content-Type', 'application/x-www-form-urlencoded');
            
            $sender->sendRequest();
            echo $sender->getResponseCode();
            $this->last_response = $sender->getResponseBody();
            $responseCode = $sender->getResponseCode();
        }

        if ($responseCode == 204) {
            return true;
        }
        return false;
        
    }
}

?>
